"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Authorization_1 = __importDefault(require("./Authorization"));
const Feature_1 = __importDefault(require("./Feature"));
const ObjectModule_1 = __importDefault(require("./ObjectModule"));
const WarrantModule_1 = __importDefault(require("./WarrantModule"));
const ObjectType_1 = require("../types/ObjectType");
class PricingTier {
    constructor(pricingTierId, meta) {
        this.pricingTierId = pricingTierId;
        this.meta = meta;
    }
    //
    // Static methods
    //
    static create(pricingTier = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield ObjectModule_1.default.create({
                object: {
                    objectType: ObjectType_1.ObjectType.PricingTier,
                    objectId: pricingTier.pricingTierId,
                },
                meta: pricingTier.meta,
            }, options);
            return new PricingTier(response.objectId, response.meta);
        });
    }
    static get(pricingTierId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield ObjectModule_1.default.get({
                objectType: ObjectType_1.ObjectType.PricingTier,
                objectId: pricingTierId,
            }, options);
            return new PricingTier(response.objectId, response.meta);
        });
    }
    static update(pricingTierId, params, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield ObjectModule_1.default.update({
                objectType: ObjectType_1.ObjectType.PricingTier,
                objectId: pricingTierId,
            }, { meta: params.meta }, options);
            return new PricingTier(response.objectId, response.meta);
        });
    }
    static delete(pricingTierId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return ObjectModule_1.default.delete({
                objectType: ObjectType_1.ObjectType.PricingTier,
                objectId: pricingTierId,
            }, options);
        });
    }
    static listPricingTiers(listParams = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield ObjectModule_1.default.list(Object.assign({ objectType: ObjectType_1.ObjectType.PricingTier }, listParams), options);
                const pricingTiers = response.results.map((object) => new PricingTier(object.objectId, object.meta));
                return Object.assign(Object.assign({}, response), { results: pricingTiers });
            }
            catch (e) {
                throw e;
            }
        });
    }
    static listPricingTiersForTenant(tenantId, listParams = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const queryResponse = yield WarrantModule_1.default.query(`select pricing-tier where tenant:${tenantId} is *`, listParams, options);
                const pricingTiers = queryResponse.results.map((queryResult) => new PricingTier(queryResult.objectId, queryResult.meta));
                return Object.assign(Object.assign({}, queryResponse), { results: pricingTiers });
            }
            catch (e) {
                throw e;
            }
        });
    }
    static assignPricingTierToTenant(tenantId, pricingTierId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.create({
                object: {
                    objectType: ObjectType_1.ObjectType.PricingTier,
                    objectId: pricingTierId,
                },
                relation: "member",
                subject: {
                    objectType: ObjectType_1.ObjectType.Tenant,
                    objectId: tenantId,
                }
            }, options);
        });
    }
    static removePricingTierFromTenant(tenantId, pricingTierId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.delete({
                object: {
                    objectType: ObjectType_1.ObjectType.PricingTier,
                    objectId: pricingTierId,
                },
                relation: "member",
                subject: {
                    objectType: ObjectType_1.ObjectType.Tenant,
                    objectId: tenantId,
                }
            }, options);
        });
    }
    static listPricingTiersForUser(userId, listParams = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const queryResponse = yield WarrantModule_1.default.query(`select pricing-tier where user:${userId} is *`, listParams, options);
                const pricingTiers = queryResponse.results.map((queryResult) => new PricingTier(queryResult.objectId, queryResult.meta));
                return Object.assign(Object.assign({}, queryResponse), { results: pricingTiers });
            }
            catch (e) {
                throw e;
            }
        });
    }
    static assignPricingTierToUser(userId, pricingTierId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.create({
                object: {
                    objectType: ObjectType_1.ObjectType.PricingTier,
                    objectId: pricingTierId,
                },
                relation: "member",
                subject: {
                    objectType: ObjectType_1.ObjectType.User,
                    objectId: userId,
                }
            }, options);
        });
    }
    static removePricingTierFromUser(userId, pricingTierId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.delete({
                object: {
                    objectType: ObjectType_1.ObjectType.PricingTier,
                    objectId: pricingTierId,
                },
                relation: "member",
                subject: {
                    objectType: ObjectType_1.ObjectType.User,
                    objectId: userId,
                }
            }, options);
        });
    }
    // Instance methods
    listFeatures(listParams = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Feature_1.default.listFeaturesForPricingTier(this.pricingTierId, listParams, options);
        });
    }
    assignFeature(featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Feature_1.default.assignFeatureToPricingTier(this.pricingTierId, featureId, options);
        });
    }
    removeFeature(featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Feature_1.default.removeFeatureFromPricingTier(this.pricingTierId, featureId, options);
        });
    }
    hasFeature(params, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Authorization_1.default.hasFeature({ featureId: params.featureId, subject: { objectType: ObjectType_1.ObjectType.PricingTier, objectId: this.pricingTierId }, context: params.context }, options);
        });
    }
    // WarrantObject methods
    getObjectType() {
        return ObjectType_1.ObjectType.PricingTier;
    }
    getObjectId() {
        return this.pricingTierId;
    }
}
exports.default = PricingTier;
