"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Authorization_1 = __importDefault(require("./Authorization"));
const ObjectModule_1 = __importDefault(require("./ObjectModule"));
const Permission_1 = __importDefault(require("./Permission"));
const WarrantModule_1 = __importDefault(require("./WarrantModule"));
const ObjectType_1 = require("../types/ObjectType");
class Role {
    constructor(roleId, meta) {
        this.roleId = roleId;
        this.meta = meta;
    }
    //
    // Static methods
    //
    static create(role = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield ObjectModule_1.default.create({
                object: {
                    objectType: ObjectType_1.ObjectType.Role,
                    objectId: role.roleId,
                },
                meta: role.meta,
            }, options);
            return new Role(response.objectId, response.meta);
        });
    }
    static get(roleId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield ObjectModule_1.default.get({
                objectType: ObjectType_1.ObjectType.Role,
                objectId: roleId,
            }, options);
            return new Role(response.objectId, response.meta);
        });
    }
    static update(roleId, params, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield ObjectModule_1.default.update({
                objectType: ObjectType_1.ObjectType.Role,
                objectId: roleId,
            }, { meta: params.meta }, options);
            return new Role(response.objectId, response.meta);
        });
    }
    static delete(roleId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return ObjectModule_1.default.delete({
                objectType: ObjectType_1.ObjectType.Role,
                objectId: roleId,
            }, options);
        });
    }
    static listRoles(listParams = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield ObjectModule_1.default.list(Object.assign({ objectType: ObjectType_1.ObjectType.Role }, listParams), options);
            const roles = response.results.map((object) => new Role(object.objectId, object.meta));
            return Object.assign(Object.assign({}, response), { results: roles });
        });
    }
    static listRolesForUser(userId, listParams = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const queryResponse = yield WarrantModule_1.default.query(`select role where user:${userId} is *`, listParams, options);
            const roles = queryResponse.results.map((queryResult) => new Role(queryResult.objectId, queryResult.meta));
            return Object.assign(Object.assign({}, queryResponse), { results: roles });
        });
    }
    static assignRoleToUser(userId, roleId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.create({
                object: {
                    objectType: ObjectType_1.ObjectType.Role,
                    objectId: roleId,
                },
                relation: "member",
                subject: {
                    objectType: ObjectType_1.ObjectType.User,
                    objectId: userId,
                }
            }, options);
        });
    }
    static removeRoleFromUser(userId, roleId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.delete({
                object: {
                    objectType: ObjectType_1.ObjectType.Role,
                    objectId: roleId,
                },
                relation: "member",
                subject: {
                    objectType: ObjectType_1.ObjectType.User,
                    objectId: userId,
                }
            }, options);
        });
    }
    // Instance methods
    listPermissions(listParams = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Permission_1.default.listPermissionsForRole(this.roleId, listParams, options);
        });
    }
    assignPermission(permissionId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Permission_1.default.assignPermissionToRole(this.roleId, permissionId, options);
        });
    }
    removePermission(permissionId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Permission_1.default.removePermissionFromRole(this.roleId, permissionId, options);
        });
    }
    hasPermission(params, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Authorization_1.default.hasPermission({ permissionId: params.permissionId, subject: { objectType: ObjectType_1.ObjectType.Role, objectId: this.roleId }, context: params.context }, options);
        });
    }
    // WarrantObject methods
    getObjectType() {
        return ObjectType_1.ObjectType.Role;
    }
    getObjectId() {
        return this.roleId;
    }
}
exports.default = Role;
