import { ListFeatureParams, ListResponse, WarrantObject, ListPermissionParams, ListPricingTierParams, ListRoleParams, CreateUserParams, DeleteUserParams, ListUserParams, UpdateUserParams, ListTenantParams, Warrant, PermissionCheckParams, FeatureCheckParams } from "../types";
import Feature from "./Feature";
import Permission from "./Permission";
import PricingTier from "./PricingTier";
import Role from "./Role";
import Tenant from "./Tenant";
import { WarrantRequestOptions } from "../types/WarrantRequestOptions";
export default class User implements WarrantObject {
    userId: string;
    meta?: {
        [key: string]: any;
    };
    constructor(userId: string, meta: {
        [key: string]: any;
    });
    static create(params?: CreateUserParams, options?: WarrantRequestOptions): Promise<User>;
    static batchCreate(params: CreateUserParams[], options?: WarrantRequestOptions): Promise<User[]>;
    static get(userId: string, options?: WarrantRequestOptions): Promise<User>;
    static update(userId: string, params: UpdateUserParams, options?: WarrantRequestOptions): Promise<User>;
    static delete(userId: string, options?: WarrantRequestOptions): Promise<string>;
    static batchDelete(params: DeleteUserParams[], options?: WarrantRequestOptions): Promise<string>;
    static listUsers(listParams?: ListUserParams, options?: WarrantRequestOptions): Promise<ListResponse<User>>;
    static listUsersForTenant(tenantId: string, listParams?: ListUserParams, options?: WarrantRequestOptions): Promise<ListResponse<User>>;
    static assignUserToTenant(tenantId: string, userId: string, role: string, options?: WarrantRequestOptions): Promise<Warrant>;
    static removeUserFromTenant(tenantId: string, userId: string, role: string, options?: WarrantRequestOptions): Promise<string>;
    listTenants(listParams?: ListTenantParams, options?: WarrantRequestOptions): Promise<ListResponse<Tenant>>;
    listRoles(listParams?: ListRoleParams, options?: WarrantRequestOptions): Promise<ListResponse<Role>>;
    assignRole(roleId: string, options?: WarrantRequestOptions): Promise<Warrant>;
    removeRole(roleId: string, options?: WarrantRequestOptions): Promise<string>;
    listPermissions(listParams?: ListPermissionParams, options?: WarrantRequestOptions): Promise<ListResponse<Permission>>;
    assignPermission(permissionId: string, options?: WarrantRequestOptions): Promise<Warrant>;
    removePermission(permissionId: string, options?: WarrantRequestOptions): Promise<string>;
    hasPermission(params: PermissionCheckParams, options?: WarrantRequestOptions): Promise<boolean>;
    listPricingTiers(listParams?: ListPricingTierParams, options?: WarrantRequestOptions): Promise<ListResponse<PricingTier>>;
    assignPricingTier(pricingTierId: string, options?: WarrantRequestOptions): Promise<Warrant>;
    removePricingTier(pricingTierId: string, options?: WarrantRequestOptions): Promise<string>;
    listFeatures(listParams?: ListFeatureParams, options?: WarrantRequestOptions): Promise<ListResponse<Feature>>;
    assignFeature(featureId: string, options?: WarrantRequestOptions): Promise<Warrant>;
    removeFeature(featureId: string, options?: WarrantRequestOptions): Promise<string>;
    hasFeature(params: FeatureCheckParams, options?: WarrantRequestOptions): Promise<boolean>;
    getObjectType(): string;
    getObjectId(): string;
}
